"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from documenso_sdk import models, utils
from documenso_sdk._hooks import HookContext
from documenso_sdk.types import OptionalNullable, UNSET
from documenso_sdk.utils import get_security_from_env
from documenso_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional


class Embedding(BaseSDK):
    def embedding_presign_create_embedding_presign_token(
        self,
        *,
        expires_in: Optional[float] = 60,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EmbeddingPresignCreateEmbeddingPresignTokenResponse:
        r"""Create embedding presign token

        Creates a presign token for embedding operations with configurable expiration time

        :param expires_in:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EmbeddingPresignCreateEmbeddingPresignTokenRequest(
            expires_in=expires_in,
        )

        req = self._build_request(
            method="POST",
            path="/embedding/create-presign-token",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.EmbeddingPresignCreateEmbeddingPresignTokenRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="embeddingPresign-createEmbeddingPresignToken",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.EmbeddingPresignCreateEmbeddingPresignTokenResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignCreateEmbeddingPresignTokenBadRequestErrorData,
                http_res,
            )
            raise models.EmbeddingPresignCreateEmbeddingPresignTokenBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignCreateEmbeddingPresignTokenUnauthorizedErrorData,
                http_res,
            )
            raise models.EmbeddingPresignCreateEmbeddingPresignTokenUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignCreateEmbeddingPresignTokenForbiddenErrorData,
                http_res,
            )
            raise models.EmbeddingPresignCreateEmbeddingPresignTokenForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignCreateEmbeddingPresignTokenInternalServerErrorData,
                http_res,
            )
            raise models.EmbeddingPresignCreateEmbeddingPresignTokenInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def embedding_presign_create_embedding_presign_token_async(
        self,
        *,
        expires_in: Optional[float] = 60,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EmbeddingPresignCreateEmbeddingPresignTokenResponse:
        r"""Create embedding presign token

        Creates a presign token for embedding operations with configurable expiration time

        :param expires_in:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EmbeddingPresignCreateEmbeddingPresignTokenRequest(
            expires_in=expires_in,
        )

        req = self._build_request_async(
            method="POST",
            path="/embedding/create-presign-token",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.EmbeddingPresignCreateEmbeddingPresignTokenRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="embeddingPresign-createEmbeddingPresignToken",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.EmbeddingPresignCreateEmbeddingPresignTokenResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignCreateEmbeddingPresignTokenBadRequestErrorData,
                http_res,
            )
            raise models.EmbeddingPresignCreateEmbeddingPresignTokenBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignCreateEmbeddingPresignTokenUnauthorizedErrorData,
                http_res,
            )
            raise models.EmbeddingPresignCreateEmbeddingPresignTokenUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignCreateEmbeddingPresignTokenForbiddenErrorData,
                http_res,
            )
            raise models.EmbeddingPresignCreateEmbeddingPresignTokenForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignCreateEmbeddingPresignTokenInternalServerErrorData,
                http_res,
            )
            raise models.EmbeddingPresignCreateEmbeddingPresignTokenInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def embedding_presign_verify_embedding_presign_token(
        self,
        *,
        token: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EmbeddingPresignVerifyEmbeddingPresignTokenResponse:
        r"""Verify embedding presign token

        Verifies a presign token for embedding operations and returns the associated API token

        :param token:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EmbeddingPresignVerifyEmbeddingPresignTokenRequest(
            token=token,
        )

        req = self._build_request(
            method="POST",
            path="/embedding/verify-presign-token",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.EmbeddingPresignVerifyEmbeddingPresignTokenRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="embeddingPresign-verifyEmbeddingPresignToken",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.EmbeddingPresignVerifyEmbeddingPresignTokenResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignVerifyEmbeddingPresignTokenBadRequestErrorData,
                http_res,
            )
            raise models.EmbeddingPresignVerifyEmbeddingPresignTokenBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignVerifyEmbeddingPresignTokenUnauthorizedErrorData,
                http_res,
            )
            raise models.EmbeddingPresignVerifyEmbeddingPresignTokenUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignVerifyEmbeddingPresignTokenForbiddenErrorData,
                http_res,
            )
            raise models.EmbeddingPresignVerifyEmbeddingPresignTokenForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignVerifyEmbeddingPresignTokenInternalServerErrorData,
                http_res,
            )
            raise models.EmbeddingPresignVerifyEmbeddingPresignTokenInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def embedding_presign_verify_embedding_presign_token_async(
        self,
        *,
        token: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EmbeddingPresignVerifyEmbeddingPresignTokenResponse:
        r"""Verify embedding presign token

        Verifies a presign token for embedding operations and returns the associated API token

        :param token:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EmbeddingPresignVerifyEmbeddingPresignTokenRequest(
            token=token,
        )

        req = self._build_request_async(
            method="POST",
            path="/embedding/verify-presign-token",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.EmbeddingPresignVerifyEmbeddingPresignTokenRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="embeddingPresign-verifyEmbeddingPresignToken",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.EmbeddingPresignVerifyEmbeddingPresignTokenResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignVerifyEmbeddingPresignTokenBadRequestErrorData,
                http_res,
            )
            raise models.EmbeddingPresignVerifyEmbeddingPresignTokenBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignVerifyEmbeddingPresignTokenUnauthorizedErrorData,
                http_res,
            )
            raise models.EmbeddingPresignVerifyEmbeddingPresignTokenUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignVerifyEmbeddingPresignTokenForbiddenErrorData,
                http_res,
            )
            raise models.EmbeddingPresignVerifyEmbeddingPresignTokenForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.EmbeddingPresignVerifyEmbeddingPresignTokenInternalServerErrorData,
                http_res,
            )
            raise models.EmbeddingPresignVerifyEmbeddingPresignTokenInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
import httpx
import pydantic
from pydantic import model_serializer
from typing import Any, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class FieldUpdateTemplateFieldsTypeDropdownRequest1(str, Enum):
    DROPDOWN = "DROPDOWN"


class FieldUpdateTemplateFieldsTypeDropdownRequest2(str, Enum):
    DROPDOWN = "dropdown"


class FieldUpdateTemplateFieldsValueDropdownTypedDict(TypedDict):
    value: str


class FieldUpdateTemplateFieldsValueDropdown(BaseModel):
    value: str


class FieldUpdateTemplateFieldsFieldMetaDropdownRequestTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeDropdownRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldUpdateTemplateFieldsValueDropdownTypedDict]]
    default_value: NotRequired[str]


class FieldUpdateTemplateFieldsFieldMetaDropdownRequest(BaseModel):
    type: FieldUpdateTemplateFieldsTypeDropdownRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldUpdateTemplateFieldsValueDropdown]] = None

    default_value: Annotated[Optional[str], pydantic.Field(alias="defaultValue")] = None


class FieldUpdateTemplateFieldsFieldDropdownTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeDropdownRequest1
    id: float
    field_meta: NotRequired[FieldUpdateTemplateFieldsFieldMetaDropdownRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateTemplateFieldsFieldDropdown(BaseModel):
    type: FieldUpdateTemplateFieldsTypeDropdownRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateTemplateFieldsFieldMetaDropdownRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateTemplateFieldsTypeCheckboxRequest1(str, Enum):
    CHECKBOX = "CHECKBOX"


class FieldUpdateTemplateFieldsTypeCheckboxRequest2(str, Enum):
    CHECKBOX = "checkbox"


class FieldUpdateTemplateFieldsValueCheckboxTypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class FieldUpdateTemplateFieldsValueCheckbox(BaseModel):
    id: float

    checked: bool

    value: str


class FieldUpdateTemplateFieldsDirectionCheckbox(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class FieldUpdateTemplateFieldsFieldMetaCheckboxRequestTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeCheckboxRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldUpdateTemplateFieldsValueCheckboxTypedDict]]
    validation_rule: NotRequired[str]
    validation_length: NotRequired[float]
    direction: NotRequired[FieldUpdateTemplateFieldsDirectionCheckbox]


class FieldUpdateTemplateFieldsFieldMetaCheckboxRequest(BaseModel):
    type: FieldUpdateTemplateFieldsTypeCheckboxRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldUpdateTemplateFieldsValueCheckbox]] = None

    validation_rule: Annotated[
        Optional[str], pydantic.Field(alias="validationRule")
    ] = None

    validation_length: Annotated[
        Optional[float], pydantic.Field(alias="validationLength")
    ] = None

    direction: Optional[FieldUpdateTemplateFieldsDirectionCheckbox] = (
        FieldUpdateTemplateFieldsDirectionCheckbox.VERTICAL
    )


class FieldUpdateTemplateFieldsFieldCheckboxTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeCheckboxRequest1
    id: float
    field_meta: NotRequired[FieldUpdateTemplateFieldsFieldMetaCheckboxRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateTemplateFieldsFieldCheckbox(BaseModel):
    type: FieldUpdateTemplateFieldsTypeCheckboxRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateTemplateFieldsFieldMetaCheckboxRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateTemplateFieldsTypeRadioRequest1(str, Enum):
    RADIO = "RADIO"


class FieldUpdateTemplateFieldsTypeRadioRequest2(str, Enum):
    RADIO = "radio"


class FieldUpdateTemplateFieldsValueRadioTypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class FieldUpdateTemplateFieldsValueRadio(BaseModel):
    id: float

    checked: bool

    value: str


class FieldUpdateTemplateFieldsDirectionRadio(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class FieldUpdateTemplateFieldsFieldMetaRadioRequestTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeRadioRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldUpdateTemplateFieldsValueRadioTypedDict]]
    direction: NotRequired[FieldUpdateTemplateFieldsDirectionRadio]


class FieldUpdateTemplateFieldsFieldMetaRadioRequest(BaseModel):
    type: FieldUpdateTemplateFieldsTypeRadioRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldUpdateTemplateFieldsValueRadio]] = None

    direction: Optional[FieldUpdateTemplateFieldsDirectionRadio] = (
        FieldUpdateTemplateFieldsDirectionRadio.VERTICAL
    )


class FieldUpdateTemplateFieldsFieldRadioTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeRadioRequest1
    id: float
    field_meta: NotRequired[FieldUpdateTemplateFieldsFieldMetaRadioRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateTemplateFieldsFieldRadio(BaseModel):
    type: FieldUpdateTemplateFieldsTypeRadioRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateTemplateFieldsFieldMetaRadioRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateTemplateFieldsTypeNumberRequest1(str, Enum):
    NUMBER = "NUMBER"


class FieldUpdateTemplateFieldsTypeNumberRequest2(str, Enum):
    NUMBER = "number"


class FieldUpdateTemplateFieldsTextAlignNumber(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateTemplateFieldsVerticalAlignNumber(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class FieldUpdateTemplateFieldsFieldMetaNumberRequestTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeNumberRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    number_format: NotRequired[Nullable[str]]
    value: NotRequired[str]
    min_value: NotRequired[Nullable[float]]
    max_value: NotRequired[Nullable[float]]
    text_align: NotRequired[FieldUpdateTemplateFieldsTextAlignNumber]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[FieldUpdateTemplateFieldsVerticalAlignNumber]]


class FieldUpdateTemplateFieldsFieldMetaNumberRequest(BaseModel):
    type: FieldUpdateTemplateFieldsTypeNumberRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    number_format: Annotated[
        OptionalNullable[str], pydantic.Field(alias="numberFormat")
    ] = UNSET

    value: Optional[str] = None

    min_value: Annotated[OptionalNullable[float], pydantic.Field(alias="minValue")] = (
        UNSET
    )

    max_value: Annotated[OptionalNullable[float], pydantic.Field(alias="maxValue")] = (
        UNSET
    )

    text_align: Annotated[
        Optional[FieldUpdateTemplateFieldsTextAlignNumber],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[FieldUpdateTemplateFieldsVerticalAlignNumber],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "numberFormat",
            "value",
            "minValue",
            "maxValue",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = [
            "numberFormat",
            "minValue",
            "maxValue",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldUpdateTemplateFieldsFieldNumberTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeNumberRequest1
    id: float
    field_meta: NotRequired[FieldUpdateTemplateFieldsFieldMetaNumberRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateTemplateFieldsFieldNumber(BaseModel):
    type: FieldUpdateTemplateFieldsTypeNumberRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateTemplateFieldsFieldMetaNumberRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateTemplateFieldsTypeTextRequest1(str, Enum):
    TEXT = "TEXT"


class FieldUpdateTemplateFieldsTypeTextRequest2(str, Enum):
    TEXT = "text"


class FieldUpdateTemplateFieldsTextAlignText(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateTemplateFieldsVerticalAlignText(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class FieldUpdateTemplateFieldsFieldMetaTextRequestTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeTextRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text: NotRequired[str]
    character_limit: NotRequired[float]
    text_align: NotRequired[FieldUpdateTemplateFieldsTextAlignText]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[FieldUpdateTemplateFieldsVerticalAlignText]]


class FieldUpdateTemplateFieldsFieldMetaTextRequest(BaseModel):
    type: FieldUpdateTemplateFieldsTypeTextRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text: Optional[str] = None

    character_limit: Annotated[
        Optional[float], pydantic.Field(alias="characterLimit")
    ] = None

    text_align: Annotated[
        Optional[FieldUpdateTemplateFieldsTextAlignText],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[FieldUpdateTemplateFieldsVerticalAlignText],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "text",
            "characterLimit",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = ["lineHeight", "letterSpacing", "verticalAlign"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldUpdateTemplateFieldsFieldTextTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeTextRequest1
    id: float
    field_meta: NotRequired[FieldUpdateTemplateFieldsFieldMetaTextRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateTemplateFieldsFieldText(BaseModel):
    type: FieldUpdateTemplateFieldsTypeTextRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateTemplateFieldsFieldMetaTextRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateTemplateFieldsTypeDateRequest1(str, Enum):
    DATE = "DATE"


class FieldUpdateTemplateFieldsTypeDateRequest2(str, Enum):
    DATE = "date"


class FieldUpdateTemplateFieldsTextAlignDate(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateTemplateFieldsFieldMetaDateRequestTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeDateRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateTemplateFieldsTextAlignDate]


class FieldUpdateTemplateFieldsFieldMetaDateRequest(BaseModel):
    type: FieldUpdateTemplateFieldsTypeDateRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateTemplateFieldsTextAlignDate],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateTemplateFieldsFieldDateTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeDateRequest1
    id: float
    field_meta: NotRequired[FieldUpdateTemplateFieldsFieldMetaDateRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateTemplateFieldsFieldDate(BaseModel):
    type: FieldUpdateTemplateFieldsTypeDateRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateTemplateFieldsFieldMetaDateRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateTemplateFieldsTypeEmailRequest1(str, Enum):
    EMAIL = "EMAIL"


class FieldUpdateTemplateFieldsTypeEmailRequest2(str, Enum):
    EMAIL = "email"


class FieldUpdateTemplateFieldsTextAlignEmail(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateTemplateFieldsFieldMetaEmailRequestTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeEmailRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateTemplateFieldsTextAlignEmail]


class FieldUpdateTemplateFieldsFieldMetaEmailRequest(BaseModel):
    type: FieldUpdateTemplateFieldsTypeEmailRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateTemplateFieldsTextAlignEmail],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateTemplateFieldsFieldEmailTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeEmailRequest1
    id: float
    field_meta: NotRequired[FieldUpdateTemplateFieldsFieldMetaEmailRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateTemplateFieldsFieldEmail(BaseModel):
    type: FieldUpdateTemplateFieldsTypeEmailRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateTemplateFieldsFieldMetaEmailRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateTemplateFieldsTypeNameRequest1(str, Enum):
    NAME = "NAME"


class FieldUpdateTemplateFieldsTypeNameRequest2(str, Enum):
    NAME = "name"


class FieldUpdateTemplateFieldsTextAlignName(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateTemplateFieldsFieldMetaNameRequestTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeNameRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateTemplateFieldsTextAlignName]


class FieldUpdateTemplateFieldsFieldMetaNameRequest(BaseModel):
    type: FieldUpdateTemplateFieldsTypeNameRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateTemplateFieldsTextAlignName],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateTemplateFieldsFieldNameTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeNameRequest1
    id: float
    field_meta: NotRequired[FieldUpdateTemplateFieldsFieldMetaNameRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateTemplateFieldsFieldName(BaseModel):
    type: FieldUpdateTemplateFieldsTypeNameRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateTemplateFieldsFieldMetaNameRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateTemplateFieldsTypeInitialsRequest1(str, Enum):
    INITIALS = "INITIALS"


class FieldUpdateTemplateFieldsTypeInitialsRequest2(str, Enum):
    INITIALS = "initials"


class FieldUpdateTemplateFieldsTextAlignInitials(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateTemplateFieldsFieldMetaInitialsRequestTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeInitialsRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateTemplateFieldsTextAlignInitials]


class FieldUpdateTemplateFieldsFieldMetaInitialsRequest(BaseModel):
    type: FieldUpdateTemplateFieldsTypeInitialsRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateTemplateFieldsTextAlignInitials],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateTemplateFieldsFieldInitialsTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeInitialsRequest1
    id: float
    field_meta: NotRequired[FieldUpdateTemplateFieldsFieldMetaInitialsRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateTemplateFieldsFieldInitials(BaseModel):
    type: FieldUpdateTemplateFieldsTypeInitialsRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateTemplateFieldsFieldMetaInitialsRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateTemplateFieldsTypeFreeSignature(str, Enum):
    FREE_SIGNATURE = "FREE_SIGNATURE"


class FieldUpdateTemplateFieldsFieldFreeSignatureTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeFreeSignature
    id: float
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateTemplateFieldsFieldFreeSignature(BaseModel):
    type: FieldUpdateTemplateFieldsTypeFreeSignature

    id: float

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateTemplateFieldsTypeSignatureRequest1(str, Enum):
    SIGNATURE = "SIGNATURE"


class FieldUpdateTemplateFieldsTypeSignatureRequest2(str, Enum):
    SIGNATURE = "signature"


class FieldUpdateTemplateFieldsFieldMetaSignatureRequestTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeSignatureRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]


class FieldUpdateTemplateFieldsFieldMetaSignatureRequest(BaseModel):
    type: FieldUpdateTemplateFieldsTypeSignatureRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12


class FieldUpdateTemplateFieldsFieldSignatureTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsTypeSignatureRequest1
    id: float
    field_meta: NotRequired[FieldUpdateTemplateFieldsFieldMetaSignatureRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateTemplateFieldsFieldSignature(BaseModel):
    type: FieldUpdateTemplateFieldsTypeSignatureRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateTemplateFieldsFieldMetaSignatureRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


FieldUpdateTemplateFieldsFieldUnionTypedDict = TypeAliasType(
    "FieldUpdateTemplateFieldsFieldUnionTypedDict",
    Union[
        FieldUpdateTemplateFieldsFieldFreeSignatureTypedDict,
        FieldUpdateTemplateFieldsFieldSignatureTypedDict,
        FieldUpdateTemplateFieldsFieldInitialsTypedDict,
        FieldUpdateTemplateFieldsFieldNameTypedDict,
        FieldUpdateTemplateFieldsFieldEmailTypedDict,
        FieldUpdateTemplateFieldsFieldDateTypedDict,
        FieldUpdateTemplateFieldsFieldTextTypedDict,
        FieldUpdateTemplateFieldsFieldNumberTypedDict,
        FieldUpdateTemplateFieldsFieldRadioTypedDict,
        FieldUpdateTemplateFieldsFieldCheckboxTypedDict,
        FieldUpdateTemplateFieldsFieldDropdownTypedDict,
    ],
)


FieldUpdateTemplateFieldsFieldUnion = TypeAliasType(
    "FieldUpdateTemplateFieldsFieldUnion",
    Union[
        FieldUpdateTemplateFieldsFieldFreeSignature,
        FieldUpdateTemplateFieldsFieldSignature,
        FieldUpdateTemplateFieldsFieldInitials,
        FieldUpdateTemplateFieldsFieldName,
        FieldUpdateTemplateFieldsFieldEmail,
        FieldUpdateTemplateFieldsFieldDate,
        FieldUpdateTemplateFieldsFieldText,
        FieldUpdateTemplateFieldsFieldNumber,
        FieldUpdateTemplateFieldsFieldRadio,
        FieldUpdateTemplateFieldsFieldCheckbox,
        FieldUpdateTemplateFieldsFieldDropdown,
    ],
)


class FieldUpdateTemplateFieldsRequestTypedDict(TypedDict):
    template_id: float
    fields: List[FieldUpdateTemplateFieldsFieldUnionTypedDict]


class FieldUpdateTemplateFieldsRequest(BaseModel):
    template_id: Annotated[float, pydantic.Field(alias="templateId")]

    fields: List[FieldUpdateTemplateFieldsFieldUnion]


class FieldUpdateTemplateFieldsInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class FieldUpdateTemplateFieldsInternalServerErrorIssue(BaseModel):
    message: str


class FieldUpdateTemplateFieldsInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FieldUpdateTemplateFieldsInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class FieldUpdateTemplateFieldsInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: FieldUpdateTemplateFieldsInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: FieldUpdateTemplateFieldsInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FieldUpdateTemplateFieldsForbiddenIssueTypedDict(TypedDict):
    message: str


class FieldUpdateTemplateFieldsForbiddenIssue(BaseModel):
    message: str


class FieldUpdateTemplateFieldsForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FieldUpdateTemplateFieldsForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class FieldUpdateTemplateFieldsForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: FieldUpdateTemplateFieldsForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: FieldUpdateTemplateFieldsForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FieldUpdateTemplateFieldsUnauthorizedIssueTypedDict(TypedDict):
    message: str


class FieldUpdateTemplateFieldsUnauthorizedIssue(BaseModel):
    message: str


class FieldUpdateTemplateFieldsUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FieldUpdateTemplateFieldsUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class FieldUpdateTemplateFieldsUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: FieldUpdateTemplateFieldsUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: FieldUpdateTemplateFieldsUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FieldUpdateTemplateFieldsBadRequestIssueTypedDict(TypedDict):
    message: str


class FieldUpdateTemplateFieldsBadRequestIssue(BaseModel):
    message: str


class FieldUpdateTemplateFieldsBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FieldUpdateTemplateFieldsBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class FieldUpdateTemplateFieldsBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: FieldUpdateTemplateFieldsBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: FieldUpdateTemplateFieldsBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FieldUpdateTemplateFieldsTypeResponse(str, Enum):
    SIGNATURE = "SIGNATURE"
    FREE_SIGNATURE = "FREE_SIGNATURE"
    INITIALS = "INITIALS"
    NAME = "NAME"
    EMAIL = "EMAIL"
    DATE = "DATE"
    TEXT = "TEXT"
    NUMBER = "NUMBER"
    RADIO = "RADIO"
    CHECKBOX = "CHECKBOX"
    DROPDOWN = "DROPDOWN"


class FieldUpdateTemplateFieldsFieldMetaTypeDropdown(str, Enum):
    DROPDOWN = "dropdown"


class FieldUpdateTemplateFieldsValueResponse3TypedDict(TypedDict):
    value: str


class FieldUpdateTemplateFieldsValueResponse3(BaseModel):
    value: str


class FieldUpdateTemplateFieldsFieldMetaDropdownResponseTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsFieldMetaTypeDropdown
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldUpdateTemplateFieldsValueResponse3TypedDict]]
    default_value: NotRequired[str]


class FieldUpdateTemplateFieldsFieldMetaDropdownResponse(BaseModel):
    type: FieldUpdateTemplateFieldsFieldMetaTypeDropdown

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldUpdateTemplateFieldsValueResponse3]] = None

    default_value: Annotated[Optional[str], pydantic.Field(alias="defaultValue")] = None


class FieldUpdateTemplateFieldsFieldMetaTypeCheckbox(str, Enum):
    CHECKBOX = "checkbox"


class FieldUpdateTemplateFieldsValueResponse2TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class FieldUpdateTemplateFieldsValueResponse2(BaseModel):
    id: float

    checked: bool

    value: str


class FieldUpdateTemplateFieldsDirectionResponse2(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class FieldUpdateTemplateFieldsFieldMetaCheckboxResponseTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsFieldMetaTypeCheckbox
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldUpdateTemplateFieldsValueResponse2TypedDict]]
    validation_rule: NotRequired[str]
    validation_length: NotRequired[float]
    direction: NotRequired[FieldUpdateTemplateFieldsDirectionResponse2]


class FieldUpdateTemplateFieldsFieldMetaCheckboxResponse(BaseModel):
    type: FieldUpdateTemplateFieldsFieldMetaTypeCheckbox

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldUpdateTemplateFieldsValueResponse2]] = None

    validation_rule: Annotated[
        Optional[str], pydantic.Field(alias="validationRule")
    ] = None

    validation_length: Annotated[
        Optional[float], pydantic.Field(alias="validationLength")
    ] = None

    direction: Optional[FieldUpdateTemplateFieldsDirectionResponse2] = (
        FieldUpdateTemplateFieldsDirectionResponse2.VERTICAL
    )


class FieldUpdateTemplateFieldsFieldMetaTypeRadio(str, Enum):
    RADIO = "radio"


class FieldUpdateTemplateFieldsValueResponse1TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class FieldUpdateTemplateFieldsValueResponse1(BaseModel):
    id: float

    checked: bool

    value: str


class FieldUpdateTemplateFieldsDirectionResponse1(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class FieldUpdateTemplateFieldsFieldMetaRadioResponseTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsFieldMetaTypeRadio
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldUpdateTemplateFieldsValueResponse1TypedDict]]
    direction: NotRequired[FieldUpdateTemplateFieldsDirectionResponse1]


class FieldUpdateTemplateFieldsFieldMetaRadioResponse(BaseModel):
    type: FieldUpdateTemplateFieldsFieldMetaTypeRadio

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldUpdateTemplateFieldsValueResponse1]] = None

    direction: Optional[FieldUpdateTemplateFieldsDirectionResponse1] = (
        FieldUpdateTemplateFieldsDirectionResponse1.VERTICAL
    )


class FieldUpdateTemplateFieldsFieldMetaTypeNumber(str, Enum):
    NUMBER = "number"


class FieldUpdateTemplateFieldsTextAlignResponse6(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateTemplateFieldsVerticalAlignResponse2(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class FieldUpdateTemplateFieldsFieldMetaNumberResponseTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsFieldMetaTypeNumber
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    number_format: NotRequired[Nullable[str]]
    value: NotRequired[str]
    min_value: NotRequired[Nullable[float]]
    max_value: NotRequired[Nullable[float]]
    text_align: NotRequired[FieldUpdateTemplateFieldsTextAlignResponse6]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[
        Nullable[FieldUpdateTemplateFieldsVerticalAlignResponse2]
    ]


class FieldUpdateTemplateFieldsFieldMetaNumberResponse(BaseModel):
    type: FieldUpdateTemplateFieldsFieldMetaTypeNumber

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    number_format: Annotated[
        OptionalNullable[str], pydantic.Field(alias="numberFormat")
    ] = UNSET

    value: Optional[str] = None

    min_value: Annotated[OptionalNullable[float], pydantic.Field(alias="minValue")] = (
        UNSET
    )

    max_value: Annotated[OptionalNullable[float], pydantic.Field(alias="maxValue")] = (
        UNSET
    )

    text_align: Annotated[
        Optional[FieldUpdateTemplateFieldsTextAlignResponse6],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[FieldUpdateTemplateFieldsVerticalAlignResponse2],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "numberFormat",
            "value",
            "minValue",
            "maxValue",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = [
            "numberFormat",
            "minValue",
            "maxValue",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldUpdateTemplateFieldsFieldMetaTypeText(str, Enum):
    TEXT = "text"


class FieldUpdateTemplateFieldsTextAlignResponse5(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateTemplateFieldsVerticalAlignResponse1(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class FieldUpdateTemplateFieldsFieldMetaTextResponseTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsFieldMetaTypeText
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text: NotRequired[str]
    character_limit: NotRequired[float]
    text_align: NotRequired[FieldUpdateTemplateFieldsTextAlignResponse5]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[
        Nullable[FieldUpdateTemplateFieldsVerticalAlignResponse1]
    ]


class FieldUpdateTemplateFieldsFieldMetaTextResponse(BaseModel):
    type: FieldUpdateTemplateFieldsFieldMetaTypeText

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text: Optional[str] = None

    character_limit: Annotated[
        Optional[float], pydantic.Field(alias="characterLimit")
    ] = None

    text_align: Annotated[
        Optional[FieldUpdateTemplateFieldsTextAlignResponse5],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[FieldUpdateTemplateFieldsVerticalAlignResponse1],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "text",
            "characterLimit",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = ["lineHeight", "letterSpacing", "verticalAlign"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldUpdateTemplateFieldsFieldMetaTypeDate(str, Enum):
    DATE = "date"


class FieldUpdateTemplateFieldsTextAlignResponse4(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateTemplateFieldsFieldMetaDateResponseTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsFieldMetaTypeDate
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateTemplateFieldsTextAlignResponse4]


class FieldUpdateTemplateFieldsFieldMetaDateResponse(BaseModel):
    type: FieldUpdateTemplateFieldsFieldMetaTypeDate

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateTemplateFieldsTextAlignResponse4],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateTemplateFieldsFieldMetaTypeEmail(str, Enum):
    EMAIL = "email"


class FieldUpdateTemplateFieldsTextAlignResponse3(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateTemplateFieldsFieldMetaEmailResponseTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsFieldMetaTypeEmail
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateTemplateFieldsTextAlignResponse3]


class FieldUpdateTemplateFieldsFieldMetaEmailResponse(BaseModel):
    type: FieldUpdateTemplateFieldsFieldMetaTypeEmail

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateTemplateFieldsTextAlignResponse3],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateTemplateFieldsFieldMetaTypeName(str, Enum):
    NAME = "name"


class FieldUpdateTemplateFieldsTextAlignResponse2(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateTemplateFieldsFieldMetaNameResponseTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsFieldMetaTypeName
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateTemplateFieldsTextAlignResponse2]


class FieldUpdateTemplateFieldsFieldMetaNameResponse(BaseModel):
    type: FieldUpdateTemplateFieldsFieldMetaTypeName

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateTemplateFieldsTextAlignResponse2],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateTemplateFieldsFieldMetaTypeInitials(str, Enum):
    INITIALS = "initials"


class FieldUpdateTemplateFieldsTextAlignResponse1(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateTemplateFieldsFieldMetaInitialsResponseTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsFieldMetaTypeInitials
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateTemplateFieldsTextAlignResponse1]


class FieldUpdateTemplateFieldsFieldMetaInitialsResponse(BaseModel):
    type: FieldUpdateTemplateFieldsFieldMetaTypeInitials

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateTemplateFieldsTextAlignResponse1],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateTemplateFieldsFieldMetaTypeSignature(str, Enum):
    SIGNATURE = "signature"


class FieldUpdateTemplateFieldsFieldMetaSignatureResponseTypedDict(TypedDict):
    type: FieldUpdateTemplateFieldsFieldMetaTypeSignature
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]


class FieldUpdateTemplateFieldsFieldMetaSignatureResponse(BaseModel):
    type: FieldUpdateTemplateFieldsFieldMetaTypeSignature

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12


FieldUpdateTemplateFieldsFieldMetaUnionTypedDict = TypeAliasType(
    "FieldUpdateTemplateFieldsFieldMetaUnionTypedDict",
    Union[
        FieldUpdateTemplateFieldsFieldMetaSignatureResponseTypedDict,
        FieldUpdateTemplateFieldsFieldMetaInitialsResponseTypedDict,
        FieldUpdateTemplateFieldsFieldMetaNameResponseTypedDict,
        FieldUpdateTemplateFieldsFieldMetaEmailResponseTypedDict,
        FieldUpdateTemplateFieldsFieldMetaDateResponseTypedDict,
        FieldUpdateTemplateFieldsFieldMetaRadioResponseTypedDict,
        FieldUpdateTemplateFieldsFieldMetaDropdownResponseTypedDict,
        FieldUpdateTemplateFieldsFieldMetaCheckboxResponseTypedDict,
        FieldUpdateTemplateFieldsFieldMetaTextResponseTypedDict,
        FieldUpdateTemplateFieldsFieldMetaNumberResponseTypedDict,
    ],
)


FieldUpdateTemplateFieldsFieldMetaUnion = TypeAliasType(
    "FieldUpdateTemplateFieldsFieldMetaUnion",
    Union[
        FieldUpdateTemplateFieldsFieldMetaSignatureResponse,
        FieldUpdateTemplateFieldsFieldMetaInitialsResponse,
        FieldUpdateTemplateFieldsFieldMetaNameResponse,
        FieldUpdateTemplateFieldsFieldMetaEmailResponse,
        FieldUpdateTemplateFieldsFieldMetaDateResponse,
        FieldUpdateTemplateFieldsFieldMetaRadioResponse,
        FieldUpdateTemplateFieldsFieldMetaDropdownResponse,
        FieldUpdateTemplateFieldsFieldMetaCheckboxResponse,
        FieldUpdateTemplateFieldsFieldMetaTextResponse,
        FieldUpdateTemplateFieldsFieldMetaNumberResponse,
    ],
)


class FieldUpdateTemplateFieldsFieldResponseTypedDict(TypedDict):
    envelope_id: str
    envelope_item_id: str
    type: FieldUpdateTemplateFieldsTypeResponse
    id: float
    secondary_id: str
    recipient_id: float
    page: float
    position_x: Any
    position_y: Any
    width: Any
    height: Any
    custom_text: str
    inserted: bool
    field_meta: Nullable[FieldUpdateTemplateFieldsFieldMetaUnionTypedDict]
    document_id: NotRequired[Nullable[float]]
    template_id: NotRequired[Nullable[float]]


class FieldUpdateTemplateFieldsFieldResponse(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]

    type: FieldUpdateTemplateFieldsTypeResponse

    id: float

    secondary_id: Annotated[str, pydantic.Field(alias="secondaryId")]

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page: float

    position_x: Annotated[Any, pydantic.Field(alias="positionX")]

    position_y: Annotated[Any, pydantic.Field(alias="positionY")]

    width: Any

    height: Any

    custom_text: Annotated[str, pydantic.Field(alias="customText")]

    inserted: bool

    field_meta: Annotated[
        Nullable[FieldUpdateTemplateFieldsFieldMetaUnion],
        pydantic.Field(alias="fieldMeta"),
    ]

    document_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="documentId")
    ] = UNSET

    template_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="templateId")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["documentId", "templateId"]
        nullable_fields = ["fieldMeta", "documentId", "templateId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldUpdateTemplateFieldsResponseTypedDict(TypedDict):
    r"""Successful response"""

    fields: List[FieldUpdateTemplateFieldsFieldResponseTypedDict]


class FieldUpdateTemplateFieldsResponse(BaseModel):
    r"""Successful response"""

    fields: List[FieldUpdateTemplateFieldsFieldResponse]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
import httpx
from typing import List, Optional
from typing_extensions import TypedDict


class EnvelopeAttachmentUpdateDataTypedDict(TypedDict):
    label: str
    data: str


class EnvelopeAttachmentUpdateData(BaseModel):
    label: str

    data: str


class EnvelopeAttachmentUpdateRequestTypedDict(TypedDict):
    id: str
    data: EnvelopeAttachmentUpdateDataTypedDict


class EnvelopeAttachmentUpdateRequest(BaseModel):
    id: str

    data: EnvelopeAttachmentUpdateData


class EnvelopeAttachmentUpdateInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class EnvelopeAttachmentUpdateInternalServerErrorIssue(BaseModel):
    message: str


class EnvelopeAttachmentUpdateInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeAttachmentUpdateInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeAttachmentUpdateInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: EnvelopeAttachmentUpdateInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeAttachmentUpdateInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeAttachmentUpdateForbiddenIssueTypedDict(TypedDict):
    message: str


class EnvelopeAttachmentUpdateForbiddenIssue(BaseModel):
    message: str


class EnvelopeAttachmentUpdateForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeAttachmentUpdateForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeAttachmentUpdateForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: EnvelopeAttachmentUpdateForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeAttachmentUpdateForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeAttachmentUpdateUnauthorizedIssueTypedDict(TypedDict):
    message: str


class EnvelopeAttachmentUpdateUnauthorizedIssue(BaseModel):
    message: str


class EnvelopeAttachmentUpdateUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeAttachmentUpdateUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeAttachmentUpdateUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: EnvelopeAttachmentUpdateUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeAttachmentUpdateUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeAttachmentUpdateBadRequestIssueTypedDict(TypedDict):
    message: str


class EnvelopeAttachmentUpdateBadRequestIssue(BaseModel):
    message: str


class EnvelopeAttachmentUpdateBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeAttachmentUpdateBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeAttachmentUpdateBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: EnvelopeAttachmentUpdateBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeAttachmentUpdateBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeAttachmentUpdateResponseTypedDict(TypedDict):
    r"""Successful response"""

    success: bool


class EnvelopeAttachmentUpdateResponse(BaseModel):
    r"""Successful response"""

    success: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
import httpx
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TemplateCreateTemplateDirectLinkRequestTypedDict(TypedDict):
    template_id: float
    direct_recipient_id: NotRequired[float]


class TemplateCreateTemplateDirectLinkRequest(BaseModel):
    template_id: Annotated[float, pydantic.Field(alias="templateId")]

    direct_recipient_id: Annotated[
        Optional[float], pydantic.Field(alias="directRecipientId")
    ] = None


class TemplateCreateTemplateDirectLinkInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class TemplateCreateTemplateDirectLinkInternalServerErrorIssue(BaseModel):
    message: str


class TemplateCreateTemplateDirectLinkInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateCreateTemplateDirectLinkInternalServerErrorIssue]] = (
        None
    )


@dataclass(unsafe_hash=True)
class TemplateCreateTemplateDirectLinkInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: TemplateCreateTemplateDirectLinkInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateCreateTemplateDirectLinkInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateCreateTemplateDirectLinkForbiddenIssueTypedDict(TypedDict):
    message: str


class TemplateCreateTemplateDirectLinkForbiddenIssue(BaseModel):
    message: str


class TemplateCreateTemplateDirectLinkForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateCreateTemplateDirectLinkForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateCreateTemplateDirectLinkForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: TemplateCreateTemplateDirectLinkForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateCreateTemplateDirectLinkForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateCreateTemplateDirectLinkUnauthorizedIssueTypedDict(TypedDict):
    message: str


class TemplateCreateTemplateDirectLinkUnauthorizedIssue(BaseModel):
    message: str


class TemplateCreateTemplateDirectLinkUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateCreateTemplateDirectLinkUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateCreateTemplateDirectLinkUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: TemplateCreateTemplateDirectLinkUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateCreateTemplateDirectLinkUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateCreateTemplateDirectLinkBadRequestIssueTypedDict(TypedDict):
    message: str


class TemplateCreateTemplateDirectLinkBadRequestIssue(BaseModel):
    message: str


class TemplateCreateTemplateDirectLinkBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateCreateTemplateDirectLinkBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateCreateTemplateDirectLinkBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: TemplateCreateTemplateDirectLinkBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateCreateTemplateDirectLinkBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateCreateTemplateDirectLinkResponseTypedDict(TypedDict):
    r"""Successful response"""

    id: str
    token: str
    created_at: str
    enabled: bool
    direct_template_recipient_id: float
    envelope_id: str
    template_id: float


class TemplateCreateTemplateDirectLinkResponse(BaseModel):
    r"""Successful response"""

    id: str

    token: str

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    enabled: bool

    direct_template_recipient_id: Annotated[
        float, pydantic.Field(alias="directTemplateRecipientId")
    ]

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    template_id: Annotated[float, pydantic.Field(alias="templateId")]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
import httpx
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, TypedDict


class EnvelopeItemDeleteRequestTypedDict(TypedDict):
    envelope_id: str
    envelope_item_id: str


class EnvelopeItemDeleteRequest(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]


class EnvelopeItemDeleteInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class EnvelopeItemDeleteInternalServerErrorIssue(BaseModel):
    message: str


class EnvelopeItemDeleteInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeItemDeleteInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeItemDeleteInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: EnvelopeItemDeleteInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeItemDeleteInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeItemDeleteForbiddenIssueTypedDict(TypedDict):
    message: str


class EnvelopeItemDeleteForbiddenIssue(BaseModel):
    message: str


class EnvelopeItemDeleteForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeItemDeleteForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeItemDeleteForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: EnvelopeItemDeleteForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeItemDeleteForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeItemDeleteUnauthorizedIssueTypedDict(TypedDict):
    message: str


class EnvelopeItemDeleteUnauthorizedIssue(BaseModel):
    message: str


class EnvelopeItemDeleteUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeItemDeleteUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeItemDeleteUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: EnvelopeItemDeleteUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeItemDeleteUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeItemDeleteBadRequestIssueTypedDict(TypedDict):
    message: str


class EnvelopeItemDeleteBadRequestIssue(BaseModel):
    message: str


class EnvelopeItemDeleteBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeItemDeleteBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeItemDeleteBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: EnvelopeItemDeleteBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeItemDeleteBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeItemDeleteResponseTypedDict(TypedDict):
    r"""Successful response"""

    success: bool


class EnvelopeItemDeleteResponse(BaseModel):
    r"""Successful response"""

    success: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from documenso_sdk.utils import FieldMetadata, PathParamMetadata
from enum import Enum
import httpx
import pydantic
from pydantic import model_serializer
from typing import Any, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class TemplateGetTemplateByIDRequestTypedDict(TypedDict):
    template_id: float


class TemplateGetTemplateByIDRequest(BaseModel):
    template_id: Annotated[
        float,
        pydantic.Field(alias="templateId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]


class TemplateGetTemplateByIDInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class TemplateGetTemplateByIDInternalServerErrorIssue(BaseModel):
    message: str


class TemplateGetTemplateByIDInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateGetTemplateByIDInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateGetTemplateByIDInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: TemplateGetTemplateByIDInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateGetTemplateByIDInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateGetTemplateByIDNotFoundIssueTypedDict(TypedDict):
    message: str


class TemplateGetTemplateByIDNotFoundIssue(BaseModel):
    message: str


class TemplateGetTemplateByIDNotFoundErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateGetTemplateByIDNotFoundIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateGetTemplateByIDNotFoundError(DocumensoError):
    r"""Not found"""

    data: TemplateGetTemplateByIDNotFoundErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateGetTemplateByIDNotFoundErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateGetTemplateByIDForbiddenIssueTypedDict(TypedDict):
    message: str


class TemplateGetTemplateByIDForbiddenIssue(BaseModel):
    message: str


class TemplateGetTemplateByIDForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateGetTemplateByIDForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateGetTemplateByIDForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: TemplateGetTemplateByIDForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateGetTemplateByIDForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateGetTemplateByIDUnauthorizedIssueTypedDict(TypedDict):
    message: str


class TemplateGetTemplateByIDUnauthorizedIssue(BaseModel):
    message: str


class TemplateGetTemplateByIDUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateGetTemplateByIDUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateGetTemplateByIDUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: TemplateGetTemplateByIDUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateGetTemplateByIDUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateGetTemplateByIDBadRequestIssueTypedDict(TypedDict):
    message: str


class TemplateGetTemplateByIDBadRequestIssue(BaseModel):
    message: str


class TemplateGetTemplateByIDBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateGetTemplateByIDBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateGetTemplateByIDBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: TemplateGetTemplateByIDBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateGetTemplateByIDBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateGetTemplateByIDType(str, Enum):
    PUBLIC = "PUBLIC"
    PRIVATE = "PRIVATE"


class TemplateGetTemplateByIDVisibility(str, Enum):
    EVERYONE = "EVERYONE"
    MANAGER_AND_ABOVE = "MANAGER_AND_ABOVE"
    ADMIN = "ADMIN"


class TemplateGetTemplateByIDGlobalAccessAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"


class TemplateGetTemplateByIDGlobalActionAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    PASSKEY = "PASSKEY"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"
    PASSWORD = "PASSWORD"


class TemplateGetTemplateByIDAuthOptionsTypedDict(TypedDict):
    global_access_auth: List[TemplateGetTemplateByIDGlobalAccessAuth]
    global_action_auth: List[TemplateGetTemplateByIDGlobalActionAuth]


class TemplateGetTemplateByIDAuthOptions(BaseModel):
    global_access_auth: Annotated[
        List[TemplateGetTemplateByIDGlobalAccessAuth],
        pydantic.Field(alias="globalAccessAuth"),
    ]

    global_action_auth: Annotated[
        List[TemplateGetTemplateByIDGlobalActionAuth],
        pydantic.Field(alias="globalActionAuth"),
    ]


class TemplateGetTemplateByIDTemplateDocumentDataType(str, Enum):
    S3_PATH = "S3_PATH"
    BYTES = "BYTES"
    BYTES_64 = "BYTES_64"


class TemplateGetTemplateByIDTemplateDocumentDataTypedDict(TypedDict):
    type: TemplateGetTemplateByIDTemplateDocumentDataType
    id: str
    data: str
    initial_data: str
    envelope_item_id: str


class TemplateGetTemplateByIDTemplateDocumentData(BaseModel):
    type: TemplateGetTemplateByIDTemplateDocumentDataType

    id: str

    data: str

    initial_data: Annotated[str, pydantic.Field(alias="initialData")]

    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]


class TemplateGetTemplateByIDSigningOrder(str, Enum):
    PARALLEL = "PARALLEL"
    SEQUENTIAL = "SEQUENTIAL"


class TemplateGetTemplateByIDDistributionMethod(str, Enum):
    EMAIL = "EMAIL"
    NONE = "NONE"


class TemplateGetTemplateByIDEmailSettingsTypedDict(TypedDict):
    recipient_signing_request: NotRequired[bool]
    recipient_removed: NotRequired[bool]
    recipient_signed: NotRequired[bool]
    document_pending: NotRequired[bool]
    document_completed: NotRequired[bool]
    document_deleted: NotRequired[bool]
    owner_document_completed: NotRequired[bool]


class TemplateGetTemplateByIDEmailSettings(BaseModel):
    recipient_signing_request: Annotated[
        Optional[bool], pydantic.Field(alias="recipientSigningRequest")
    ] = True

    recipient_removed: Annotated[
        Optional[bool], pydantic.Field(alias="recipientRemoved")
    ] = True

    recipient_signed: Annotated[
        Optional[bool], pydantic.Field(alias="recipientSigned")
    ] = True

    document_pending: Annotated[
        Optional[bool], pydantic.Field(alias="documentPending")
    ] = True

    document_completed: Annotated[
        Optional[bool], pydantic.Field(alias="documentCompleted")
    ] = True

    document_deleted: Annotated[
        Optional[bool], pydantic.Field(alias="documentDeleted")
    ] = True

    owner_document_completed: Annotated[
        Optional[bool], pydantic.Field(alias="ownerDocumentCompleted")
    ] = True


class TemplateGetTemplateByIDTemplateMetaTypedDict(TypedDict):
    id: str
    subject: Nullable[str]
    message: Nullable[str]
    timezone: Nullable[str]
    date_format: Nullable[str]
    signing_order: TemplateGetTemplateByIDSigningOrder
    typed_signature_enabled: bool
    upload_signature_enabled: bool
    draw_signature_enabled: bool
    allow_dictate_next_signer: bool
    distribution_method: TemplateGetTemplateByIDDistributionMethod
    redirect_url: Nullable[str]
    language: str
    email_settings: Nullable[TemplateGetTemplateByIDEmailSettingsTypedDict]
    email_id: Nullable[str]
    email_reply_to: Nullable[str]
    template_id: Nullable[float]


class TemplateGetTemplateByIDTemplateMeta(BaseModel):
    id: str

    subject: Nullable[str]

    message: Nullable[str]

    timezone: Nullable[str]

    date_format: Annotated[Nullable[str], pydantic.Field(alias="dateFormat")]

    signing_order: Annotated[
        TemplateGetTemplateByIDSigningOrder, pydantic.Field(alias="signingOrder")
    ]

    typed_signature_enabled: Annotated[
        bool, pydantic.Field(alias="typedSignatureEnabled")
    ]

    upload_signature_enabled: Annotated[
        bool, pydantic.Field(alias="uploadSignatureEnabled")
    ]

    draw_signature_enabled: Annotated[
        bool, pydantic.Field(alias="drawSignatureEnabled")
    ]

    allow_dictate_next_signer: Annotated[
        bool, pydantic.Field(alias="allowDictateNextSigner")
    ]

    distribution_method: Annotated[
        TemplateGetTemplateByIDDistributionMethod,
        pydantic.Field(alias="distributionMethod"),
    ]

    redirect_url: Annotated[Nullable[str], pydantic.Field(alias="redirectUrl")]

    language: str

    email_settings: Annotated[
        Nullable[TemplateGetTemplateByIDEmailSettings],
        pydantic.Field(alias="emailSettings"),
    ]

    email_id: Annotated[Nullable[str], pydantic.Field(alias="emailId")]

    email_reply_to: Annotated[Nullable[str], pydantic.Field(alias="emailReplyTo")]

    template_id: Annotated[Nullable[float], pydantic.Field(alias="templateId")]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "subject",
            "message",
            "timezone",
            "dateFormat",
            "redirectUrl",
            "emailSettings",
            "emailId",
            "emailReplyTo",
            "templateId",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateGetTemplateByIDDirectLinkTypedDict(TypedDict):
    id: str
    envelope_id: str
    token: str
    created_at: str
    enabled: bool
    direct_template_recipient_id: float
    template_id: float


class TemplateGetTemplateByIDDirectLink(BaseModel):
    id: str

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    token: str

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    enabled: bool

    direct_template_recipient_id: Annotated[
        float, pydantic.Field(alias="directTemplateRecipientId")
    ]

    template_id: Annotated[float, pydantic.Field(alias="templateId")]


class TemplateGetTemplateByIDUserTypedDict(TypedDict):
    id: float
    name: Nullable[str]
    email: str


class TemplateGetTemplateByIDUser(BaseModel):
    id: float

    name: Nullable[str]

    email: str

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["name"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateGetTemplateByIDRole(str, Enum):
    CC = "CC"
    SIGNER = "SIGNER"
    VIEWER = "VIEWER"
    APPROVER = "APPROVER"
    ASSISTANT = "ASSISTANT"


class TemplateGetTemplateByIDReadStatus(str, Enum):
    NOT_OPENED = "NOT_OPENED"
    OPENED = "OPENED"


class TemplateGetTemplateByIDSigningStatus(str, Enum):
    NOT_SIGNED = "NOT_SIGNED"
    SIGNED = "SIGNED"
    REJECTED = "REJECTED"


class TemplateGetTemplateByIDSendStatus(str, Enum):
    NOT_SENT = "NOT_SENT"
    SENT = "SENT"


class TemplateGetTemplateByIDAccessAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"


class TemplateGetTemplateByIDActionAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    PASSKEY = "PASSKEY"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"
    PASSWORD = "PASSWORD"
    EXPLICIT_NONE = "EXPLICIT_NONE"


class TemplateGetTemplateByIDRecipientAuthOptionsTypedDict(TypedDict):
    access_auth: List[TemplateGetTemplateByIDAccessAuth]
    action_auth: List[TemplateGetTemplateByIDActionAuth]


class TemplateGetTemplateByIDRecipientAuthOptions(BaseModel):
    access_auth: Annotated[
        List[TemplateGetTemplateByIDAccessAuth], pydantic.Field(alias="accessAuth")
    ]

    action_auth: Annotated[
        List[TemplateGetTemplateByIDActionAuth], pydantic.Field(alias="actionAuth")
    ]


class TemplateGetTemplateByIDRecipientTypedDict(TypedDict):
    envelope_id: str
    role: TemplateGetTemplateByIDRole
    read_status: TemplateGetTemplateByIDReadStatus
    signing_status: TemplateGetTemplateByIDSigningStatus
    send_status: TemplateGetTemplateByIDSendStatus
    id: float
    email: str
    name: str
    token: str
    document_deleted_at: Nullable[str]
    expired: Nullable[str]
    signed_at: Nullable[str]
    auth_options: Nullable[TemplateGetTemplateByIDRecipientAuthOptionsTypedDict]
    signing_order: Nullable[float]
    rejection_reason: Nullable[str]
    document_id: NotRequired[Nullable[float]]
    template_id: NotRequired[Nullable[float]]


class TemplateGetTemplateByIDRecipient(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    role: TemplateGetTemplateByIDRole

    read_status: Annotated[
        TemplateGetTemplateByIDReadStatus, pydantic.Field(alias="readStatus")
    ]

    signing_status: Annotated[
        TemplateGetTemplateByIDSigningStatus, pydantic.Field(alias="signingStatus")
    ]

    send_status: Annotated[
        TemplateGetTemplateByIDSendStatus, pydantic.Field(alias="sendStatus")
    ]

    id: float

    email: str

    name: str

    token: str

    document_deleted_at: Annotated[
        Nullable[str], pydantic.Field(alias="documentDeletedAt")
    ]

    expired: Nullable[str]

    signed_at: Annotated[Nullable[str], pydantic.Field(alias="signedAt")]

    auth_options: Annotated[
        Nullable[TemplateGetTemplateByIDRecipientAuthOptions],
        pydantic.Field(alias="authOptions"),
    ]

    signing_order: Annotated[Nullable[float], pydantic.Field(alias="signingOrder")]

    rejection_reason: Annotated[Nullable[str], pydantic.Field(alias="rejectionReason")]

    document_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="documentId")
    ] = UNSET

    template_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="templateId")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["documentId", "templateId"]
        nullable_fields = [
            "documentDeletedAt",
            "expired",
            "signedAt",
            "authOptions",
            "signingOrder",
            "rejectionReason",
            "documentId",
            "templateId",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateGetTemplateByIDFieldType(str, Enum):
    SIGNATURE = "SIGNATURE"
    FREE_SIGNATURE = "FREE_SIGNATURE"
    INITIALS = "INITIALS"
    NAME = "NAME"
    EMAIL = "EMAIL"
    DATE = "DATE"
    TEXT = "TEXT"
    NUMBER = "NUMBER"
    RADIO = "RADIO"
    CHECKBOX = "CHECKBOX"
    DROPDOWN = "DROPDOWN"


class TemplateGetTemplateByIDTypeDropdown(str, Enum):
    DROPDOWN = "dropdown"


class TemplateGetTemplateByIDValue3TypedDict(TypedDict):
    value: str


class TemplateGetTemplateByIDValue3(BaseModel):
    value: str


class TemplateGetTemplateByIDFieldMetaDropdownTypedDict(TypedDict):
    type: TemplateGetTemplateByIDTypeDropdown
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[TemplateGetTemplateByIDValue3TypedDict]]
    default_value: NotRequired[str]


class TemplateGetTemplateByIDFieldMetaDropdown(BaseModel):
    type: TemplateGetTemplateByIDTypeDropdown

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[TemplateGetTemplateByIDValue3]] = None

    default_value: Annotated[Optional[str], pydantic.Field(alias="defaultValue")] = None


class TemplateGetTemplateByIDTypeCheckbox(str, Enum):
    CHECKBOX = "checkbox"


class TemplateGetTemplateByIDValue2TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class TemplateGetTemplateByIDValue2(BaseModel):
    id: float

    checked: bool

    value: str


class TemplateGetTemplateByIDDirection2(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class TemplateGetTemplateByIDFieldMetaCheckboxTypedDict(TypedDict):
    type: TemplateGetTemplateByIDTypeCheckbox
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[TemplateGetTemplateByIDValue2TypedDict]]
    validation_rule: NotRequired[str]
    validation_length: NotRequired[float]
    direction: NotRequired[TemplateGetTemplateByIDDirection2]


class TemplateGetTemplateByIDFieldMetaCheckbox(BaseModel):
    type: TemplateGetTemplateByIDTypeCheckbox

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[TemplateGetTemplateByIDValue2]] = None

    validation_rule: Annotated[
        Optional[str], pydantic.Field(alias="validationRule")
    ] = None

    validation_length: Annotated[
        Optional[float], pydantic.Field(alias="validationLength")
    ] = None

    direction: Optional[TemplateGetTemplateByIDDirection2] = (
        TemplateGetTemplateByIDDirection2.VERTICAL
    )


class TemplateGetTemplateByIDTypeRadio(str, Enum):
    RADIO = "radio"


class TemplateGetTemplateByIDValue1TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class TemplateGetTemplateByIDValue1(BaseModel):
    id: float

    checked: bool

    value: str


class TemplateGetTemplateByIDDirection1(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class TemplateGetTemplateByIDFieldMetaRadioTypedDict(TypedDict):
    type: TemplateGetTemplateByIDTypeRadio
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[TemplateGetTemplateByIDValue1TypedDict]]
    direction: NotRequired[TemplateGetTemplateByIDDirection1]


class TemplateGetTemplateByIDFieldMetaRadio(BaseModel):
    type: TemplateGetTemplateByIDTypeRadio

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[TemplateGetTemplateByIDValue1]] = None

    direction: Optional[TemplateGetTemplateByIDDirection1] = (
        TemplateGetTemplateByIDDirection1.VERTICAL
    )


class TemplateGetTemplateByIDTypeNumber(str, Enum):
    NUMBER = "number"


class TemplateGetTemplateByIDTextAlign6(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateGetTemplateByIDVerticalAlign2(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class TemplateGetTemplateByIDFieldMetaNumberTypedDict(TypedDict):
    type: TemplateGetTemplateByIDTypeNumber
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    number_format: NotRequired[Nullable[str]]
    value: NotRequired[str]
    min_value: NotRequired[Nullable[float]]
    max_value: NotRequired[Nullable[float]]
    text_align: NotRequired[TemplateGetTemplateByIDTextAlign6]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[TemplateGetTemplateByIDVerticalAlign2]]


class TemplateGetTemplateByIDFieldMetaNumber(BaseModel):
    type: TemplateGetTemplateByIDTypeNumber

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    number_format: Annotated[
        OptionalNullable[str], pydantic.Field(alias="numberFormat")
    ] = UNSET

    value: Optional[str] = None

    min_value: Annotated[OptionalNullable[float], pydantic.Field(alias="minValue")] = (
        UNSET
    )

    max_value: Annotated[OptionalNullable[float], pydantic.Field(alias="maxValue")] = (
        UNSET
    )

    text_align: Annotated[
        Optional[TemplateGetTemplateByIDTextAlign6], pydantic.Field(alias="textAlign")
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[TemplateGetTemplateByIDVerticalAlign2],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "numberFormat",
            "value",
            "minValue",
            "maxValue",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = [
            "numberFormat",
            "minValue",
            "maxValue",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateGetTemplateByIDTypeText(str, Enum):
    TEXT = "text"


class TemplateGetTemplateByIDTextAlign5(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateGetTemplateByIDVerticalAlign1(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class TemplateGetTemplateByIDFieldMetaTextTypedDict(TypedDict):
    type: TemplateGetTemplateByIDTypeText
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text: NotRequired[str]
    character_limit: NotRequired[float]
    text_align: NotRequired[TemplateGetTemplateByIDTextAlign5]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[TemplateGetTemplateByIDVerticalAlign1]]


class TemplateGetTemplateByIDFieldMetaText(BaseModel):
    type: TemplateGetTemplateByIDTypeText

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text: Optional[str] = None

    character_limit: Annotated[
        Optional[float], pydantic.Field(alias="characterLimit")
    ] = None

    text_align: Annotated[
        Optional[TemplateGetTemplateByIDTextAlign5], pydantic.Field(alias="textAlign")
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[TemplateGetTemplateByIDVerticalAlign1],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "text",
            "characterLimit",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = ["lineHeight", "letterSpacing", "verticalAlign"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateGetTemplateByIDTypeDate(str, Enum):
    DATE = "date"


class TemplateGetTemplateByIDTextAlign4(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateGetTemplateByIDFieldMetaDateTypedDict(TypedDict):
    type: TemplateGetTemplateByIDTypeDate
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[TemplateGetTemplateByIDTextAlign4]


class TemplateGetTemplateByIDFieldMetaDate(BaseModel):
    type: TemplateGetTemplateByIDTypeDate

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[TemplateGetTemplateByIDTextAlign4], pydantic.Field(alias="textAlign")
    ] = None


class TemplateGetTemplateByIDTypeEmail(str, Enum):
    EMAIL = "email"


class TemplateGetTemplateByIDTextAlign3(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateGetTemplateByIDFieldMetaEmailTypedDict(TypedDict):
    type: TemplateGetTemplateByIDTypeEmail
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[TemplateGetTemplateByIDTextAlign3]


class TemplateGetTemplateByIDFieldMetaEmail(BaseModel):
    type: TemplateGetTemplateByIDTypeEmail

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[TemplateGetTemplateByIDTextAlign3], pydantic.Field(alias="textAlign")
    ] = None


class TemplateGetTemplateByIDTypeName(str, Enum):
    NAME = "name"


class TemplateGetTemplateByIDTextAlign2(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateGetTemplateByIDFieldMetaNameTypedDict(TypedDict):
    type: TemplateGetTemplateByIDTypeName
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[TemplateGetTemplateByIDTextAlign2]


class TemplateGetTemplateByIDFieldMetaName(BaseModel):
    type: TemplateGetTemplateByIDTypeName

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[TemplateGetTemplateByIDTextAlign2], pydantic.Field(alias="textAlign")
    ] = None


class TemplateGetTemplateByIDTypeInitials(str, Enum):
    INITIALS = "initials"


class TemplateGetTemplateByIDTextAlign1(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class TemplateGetTemplateByIDFieldMetaInitialsTypedDict(TypedDict):
    type: TemplateGetTemplateByIDTypeInitials
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[TemplateGetTemplateByIDTextAlign1]


class TemplateGetTemplateByIDFieldMetaInitials(BaseModel):
    type: TemplateGetTemplateByIDTypeInitials

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[TemplateGetTemplateByIDTextAlign1], pydantic.Field(alias="textAlign")
    ] = None


class TemplateGetTemplateByIDTypeSignature(str, Enum):
    SIGNATURE = "signature"


class TemplateGetTemplateByIDFieldMetaSignatureTypedDict(TypedDict):
    type: TemplateGetTemplateByIDTypeSignature
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]


class TemplateGetTemplateByIDFieldMetaSignature(BaseModel):
    type: TemplateGetTemplateByIDTypeSignature

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12


TemplateGetTemplateByIDFieldMetaUnionTypedDict = TypeAliasType(
    "TemplateGetTemplateByIDFieldMetaUnionTypedDict",
    Union[
        TemplateGetTemplateByIDFieldMetaSignatureTypedDict,
        TemplateGetTemplateByIDFieldMetaInitialsTypedDict,
        TemplateGetTemplateByIDFieldMetaNameTypedDict,
        TemplateGetTemplateByIDFieldMetaEmailTypedDict,
        TemplateGetTemplateByIDFieldMetaDateTypedDict,
        TemplateGetTemplateByIDFieldMetaRadioTypedDict,
        TemplateGetTemplateByIDFieldMetaDropdownTypedDict,
        TemplateGetTemplateByIDFieldMetaCheckboxTypedDict,
        TemplateGetTemplateByIDFieldMetaTextTypedDict,
        TemplateGetTemplateByIDFieldMetaNumberTypedDict,
    ],
)


TemplateGetTemplateByIDFieldMetaUnion = TypeAliasType(
    "TemplateGetTemplateByIDFieldMetaUnion",
    Union[
        TemplateGetTemplateByIDFieldMetaSignature,
        TemplateGetTemplateByIDFieldMetaInitials,
        TemplateGetTemplateByIDFieldMetaName,
        TemplateGetTemplateByIDFieldMetaEmail,
        TemplateGetTemplateByIDFieldMetaDate,
        TemplateGetTemplateByIDFieldMetaRadio,
        TemplateGetTemplateByIDFieldMetaDropdown,
        TemplateGetTemplateByIDFieldMetaCheckbox,
        TemplateGetTemplateByIDFieldMetaText,
        TemplateGetTemplateByIDFieldMetaNumber,
    ],
)


class TemplateGetTemplateByIDFieldTypedDict(TypedDict):
    envelope_id: str
    envelope_item_id: str
    type: TemplateGetTemplateByIDFieldType
    id: float
    secondary_id: str
    recipient_id: float
    page: float
    position_x: Any
    position_y: Any
    width: Any
    height: Any
    custom_text: str
    inserted: bool
    field_meta: Nullable[TemplateGetTemplateByIDFieldMetaUnionTypedDict]
    document_id: NotRequired[Nullable[float]]
    template_id: NotRequired[Nullable[float]]


class TemplateGetTemplateByIDField(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]

    type: TemplateGetTemplateByIDFieldType

    id: float

    secondary_id: Annotated[str, pydantic.Field(alias="secondaryId")]

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page: float

    position_x: Annotated[Any, pydantic.Field(alias="positionX")]

    position_y: Annotated[Any, pydantic.Field(alias="positionY")]

    width: Any

    height: Any

    custom_text: Annotated[str, pydantic.Field(alias="customText")]

    inserted: bool

    field_meta: Annotated[
        Nullable[TemplateGetTemplateByIDFieldMetaUnion],
        pydantic.Field(alias="fieldMeta"),
    ]

    document_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="documentId")
    ] = UNSET

    template_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="templateId")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["documentId", "templateId"]
        nullable_fields = ["fieldMeta", "documentId", "templateId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateGetTemplateByIDFolderType(str, Enum):
    DOCUMENT = "DOCUMENT"
    TEMPLATE = "TEMPLATE"


class TemplateGetTemplateByIDFolderVisibility(str, Enum):
    EVERYONE = "EVERYONE"
    MANAGER_AND_ABOVE = "MANAGER_AND_ABOVE"
    ADMIN = "ADMIN"


class TemplateGetTemplateByIDFolderTypedDict(TypedDict):
    id: str
    name: str
    type: TemplateGetTemplateByIDFolderType
    visibility: TemplateGetTemplateByIDFolderVisibility
    user_id: float
    team_id: float
    pinned: bool
    parent_id: Nullable[str]
    created_at: str
    updated_at: str


class TemplateGetTemplateByIDFolder(BaseModel):
    id: str

    name: str

    type: TemplateGetTemplateByIDFolderType

    visibility: TemplateGetTemplateByIDFolderVisibility

    user_id: Annotated[float, pydantic.Field(alias="userId")]

    team_id: Annotated[float, pydantic.Field(alias="teamId")]

    pinned: bool

    parent_id: Annotated[Nullable[str], pydantic.Field(alias="parentId")]

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["parentId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TemplateGetTemplateByIDEnvelopeItemTypedDict(TypedDict):
    id: str
    envelope_id: str


class TemplateGetTemplateByIDEnvelopeItem(BaseModel):
    id: str

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]


class TemplateGetTemplateByIDResponseTypedDict(TypedDict):
    r"""Successful response"""

    type: TemplateGetTemplateByIDType
    visibility: TemplateGetTemplateByIDVisibility
    id: float
    external_id: Nullable[str]
    title: str
    user_id: float
    team_id: float
    auth_options: Nullable[TemplateGetTemplateByIDAuthOptionsTypedDict]
    created_at: str
    updated_at: str
    public_title: str
    public_description: str
    folder_id: Nullable[str]
    envelope_id: str
    template_document_data: TemplateGetTemplateByIDTemplateDocumentDataTypedDict
    template_meta: TemplateGetTemplateByIDTemplateMetaTypedDict
    direct_link: Nullable[TemplateGetTemplateByIDDirectLinkTypedDict]
    user: TemplateGetTemplateByIDUserTypedDict
    recipients: List[TemplateGetTemplateByIDRecipientTypedDict]
    fields: List[TemplateGetTemplateByIDFieldTypedDict]
    folder: Nullable[TemplateGetTemplateByIDFolderTypedDict]
    envelope_items: List[TemplateGetTemplateByIDEnvelopeItemTypedDict]
    template_document_data_id: NotRequired[str]


class TemplateGetTemplateByIDResponse(BaseModel):
    r"""Successful response"""

    type: TemplateGetTemplateByIDType

    visibility: TemplateGetTemplateByIDVisibility

    id: float

    external_id: Annotated[Nullable[str], pydantic.Field(alias="externalId")]

    title: str

    user_id: Annotated[float, pydantic.Field(alias="userId")]

    team_id: Annotated[float, pydantic.Field(alias="teamId")]

    auth_options: Annotated[
        Nullable[TemplateGetTemplateByIDAuthOptions],
        pydantic.Field(alias="authOptions"),
    ]

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]

    public_title: Annotated[str, pydantic.Field(alias="publicTitle")]

    public_description: Annotated[str, pydantic.Field(alias="publicDescription")]

    folder_id: Annotated[Nullable[str], pydantic.Field(alias="folderId")]

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    template_document_data: Annotated[
        TemplateGetTemplateByIDTemplateDocumentData,
        pydantic.Field(alias="templateDocumentData"),
    ]

    template_meta: Annotated[
        TemplateGetTemplateByIDTemplateMeta, pydantic.Field(alias="templateMeta")
    ]

    direct_link: Annotated[
        Nullable[TemplateGetTemplateByIDDirectLink], pydantic.Field(alias="directLink")
    ]

    user: TemplateGetTemplateByIDUser

    recipients: List[TemplateGetTemplateByIDRecipient]

    fields: List[TemplateGetTemplateByIDField]

    folder: Nullable[TemplateGetTemplateByIDFolder]

    envelope_items: Annotated[
        List[TemplateGetTemplateByIDEnvelopeItem], pydantic.Field(alias="envelopeItems")
    ]

    template_document_data_id: Annotated[
        Optional[str], pydantic.Field(alias="templateDocumentDataId")
    ] = ""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["templateDocumentDataId"]
        nullable_fields = [
            "externalId",
            "authOptions",
            "folderId",
            "directLink",
            "folder",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
